%     Copyright (C) 2023 Jordi Cano. Contact: jordi.cg89@gmail.com
% 
%     This program is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see <https://www.gnu.org/licenses/>

function GeneratePopulation
%GeneratePopulation Generates a population of 'popsize' male and female
%models taking into account channel mRNA expression levels

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% set variables
reference_path      = 'mRNAlevels.mat';                % mat file containing the reference data (meand and std of mRNA expression levels)
save_path           = 'Population_50000_each.mat';     % select where the data will be saved
popsize             = 50000;                           % models per gender
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% load reference data
L                   = load(reference_path);
% Extract data
sex                 = L.sex;
mean                = L.channel_mean;
sd                  = L.channel_sd;
channels            = L.channel_names;
NC                  = length(channels);

% generate distributions for each sex and channel
pd = cell(NC,2); % lines: channel; columns: sex
for i = 1:NC
    pd{i,1} = makedist('normal','mu',mean(i,1),'sigma',sd(i,2));
    pd{i,2} = makedist('normal','mu',mean(i,2),'sigma',sd(i,2));
end

% create a matrix of protein data combining the channels above
protein_mat = zeros(popsize,2,NC); % pop, sex, channel
for i = 1:NC
    protein_mat(:,1,i) = pd{i,1}.random(popsize,1);
    protein_mat(:,2,i) = pd{i,2}.random(popsize,1);
end

% recalculate special ones
a1              = find(contains(channels,'PNaKa1'));
a3              = find(contains(channels,'PNaKa3'));
ks              = find(contains(channels,'GKs'));
calm            = find(contains(channels,'calm'));
kr              = find(contains(channels,'GKr'));
for j = 1:2
    PNaKa1      = protein_mat(:,j,a1);
    PNaKa3      = protein_mat(:,j,a3);
    PNaK        = PNaKa1./3+PNaKa3./3;
    protein_mat(:,j,a3) = PNaK(:); % Use PNaK3 as PNaK
end

% calculate the scalars according to protein data and correct special ones
scalar_mat      = zeros(popsize,2,NC); % pop, sex, channel
for i = 1:NC
    scalar_mat(:,1,i)   = protein_mat(:,1,i)./mean(i,1);
    scalar_mat(:,2,i)   = protein_mat(:,2,i)./mean(i,1); % female and male are both compared to endo male
end
scalar_mat(:,1,ks)      = (scalar_mat(:,1,ks)-1)/3+1;
scalar_mat(:,2,ks)      = (scalar_mat(:,2,ks)-1)/3+1;
scalar_mat(:,2,calm)    = (scalar_mat(:,2,calm)-1).*2./3 + 1;

% bring negative values back to 0
scalar_mat(scalar_mat<0) = 0;

for gender = 1:2
    
    for channel = 1:NC
        
        while ~isempty(find(scalar_mat(:,gender,channel)<=0.1, 1))
            where = scalar_mat(:,gender,channel)<=0.1;
            indexes = find(where);
            number  = length(indexes);
            assign_this = pd{channel,gender}.random(number,1)/mean(channel,gender);
            scalar_mat(indexes,gender,channel) = assign_this(:);
            
            % same as above
            if channel == ks
                scalar_mat(:,gender,channel) = (scalar_mat(:,gender,channel)-1)/3+1;
            elseif channel == calm && gender == 2
                scalar_mat(:,gender,channel) = (scalar_mat(:,gender,channel)-1).*2./3 + 1;
            else
            end
            
        end
        
    end
end

% save the scalars for the models
coords = {'individual: patient index','sex: 1=male, 2=female','channel'};
save(save_path,'scalar_mat','channels','coords');

end

