%     Copyright (C) 2023 Jordi Cano. Contact: jordi.cg89@gmail.com
% 
%     This program is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see <https://www.gnu.org/licenses/>

function Simulate_population()
%Simulate_population Simulates the AP models with the parameters of the
%population

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% THIS FUNCTION ASSUMES THE FOLLOWING STRUCTURE OF THE MATRIX
% M(X,:,:) = individuals; M(:,X,:) = sex; M(:,:,X) = Channels
Popmatrixpath = 'Population_50000_each.mat';
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% parameters of the simulations
executable  = 'ORdmD_sex';
BCL         = 1000;
BEATS       = 500;
SVST        = BCL*(BEATS-2);
SVP         = BCL;
CELL        = 0;
GPHASE      = 1;
REMOVE_ZEROS = 1;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% this code detects the number of processors then opens a parallel pool
myCluster=parcluster('local');
poolobj = gcp('nocreate');
if isempty(poolobj)
    poolobj = parpool(myCluster.NumWorkers);
end
%%%
% this line gets the correct bar depending on the system for file paths
if isunix; bar = '/'; else; bar = '\'; end

% load the matrix
L               = load(Popmatrixpath);

% extract important variables
CHs             = L.channels;
a1              = 8; 
C               = length(CHs);
M               = L.scalar_mat;
N               = size(M,1);

% create a list of simulation IDs assuming
maleIDs         = (1:N)';
femaleIDs       = (maleIDs+N);
allIDs          = [maleIDs; femaleIDs];

% assign variables to each ID in a big matrix to convert later
% ID, sex, all channels
inc             = 1:C;
inc             = inc(inc~=a1);
channels        = CHs(inc);
channels{a1}    = 'PNaK';
parameters1     = [ maleIDs             zeros(N,1)+CELL        ones(N,1)          zeros(N,1)+GPHASE...
    zeros(N,1)+BCL      zeros(N,1)+BEATS       zeros(N,1)+SVST    zeros(N,1)+SVP...
    squeeze(M(:,1,inc))];
parameters2     = [ femaleIDs           zeros(N,1)+CELL        ones(N,1)+1        zeros(N,1)+GPHASE...
    zeros(N,1)+BCL      zeros(N,1)+BEATS       zeros(N,1)+SVST    zeros(N,1)+SVP...
    squeeze(M(:,2,inc))];
parameters      = [parameters1; parameters2];

% clear parameter sets whose conductances are 0
if REMOVE_ZEROS
    rem         = parameters(:,9:end)==0;
    rem_rows    = sum(rem'); rem_rows = rem_rows';
    these       = find(~rem_rows);
    parameters  = parameters(these,:);
end
% convert parameter lists to commands
Ncoms           = size(parameters,1);

commands        = cell(Ncoms,2);
for i = 1:Ncoms
    commands{i,1} = ['nohup > ID_' num2str(parameters(i,1)) '.log ' '.' bar executable ' ' Double2text(parameters(i,:))]; % ' &']; % ampersand not needed
    commands{i,2} = 1;
end

% check if the files exist
alreadydone = 0;
for i = 1:Ncoms
    if allIDs(i) <= N
        g = 'm';
    else
        g = 'f';
    end
    
    thisfile    = ['res_' num2str(allIDs(i)) '_ENDO' g '_p' num2str(GPHASE) '.txt'];
    fileisthere =  exist(thisfile, 'file');
    
    if fileisthere==2
        
        f           = dir(thisfile);
        mem         = f(1).bytes;
        sizeisok    = mem > 7e5;
        
        if sizeisok
            commands{i,2} = 0;
            alreadydone = alreadydone+1;
        end
        
    end
end
disp(['Skipping ' num2str(alreadydone) ' simulations because files were already present.']);

%-
savename    = Popmatrixpath(1:end-4);
save([savename '_commands.mat'],'commands','parameters','channels'); % save commands
%-

% select only the ones that haven't yet been done
donemat     = cell2mat(commands(:,2));
donebool    = donemat == 1; % 1 = to do
commands_to_do = commands(donebool,1);

parfor kk = 1:length(commands_to_do)
    
    com = commands_to_do{kk};
    system(com);
    
end


end

